<?php
/*
Plugin Name: Shortbus
Plugin URI: http://wordpress.org/extend/plugins/shortbus/
Description: Quickly and easily manage shortcodes.
Version: 1.0.4
Author: Matt Gibbs
Author URI: http://uproot.us/
License: GPL
Copyright: Matt Gibbs
*/
define('SHORTBUS_VERSION', '1.0.4');

class Shortbus
{
    /*---------------------------------------------------------------------------------------------
     * Verify the database, check Shortbus version, add necessary hooks
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function init() {
        global $wpdb;

        // Create the table if needed
        $wpdb->query("CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}shortbus` (
        `id` int unsigned AUTO_INCREMENT PRIMARY KEY,
        `name` varchar(128),
        `content` mediumtext)");

        // Update the wp_option value
        $db_version = get_option('shortbus_version');
        if (false === $db_version) {
            add_option('shortbus_version', SHORTBUS_VERSION);
        }
        elseif (version_compare($db_version, SHORTBUS_VERSION, '<')) {
            update_option('shortbus_version', SHORTBUS_VERSION);
        }

        // Add hooks
        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('admin_head', array($this, 'admin_scripts'));
        add_shortcode('shortbus', array($this, 'shortcode'));
        add_action('wp_ajax_shortbus', array($this, 'handle_ajax'));
    }

    /*---------------------------------------------------------------------------------------------
     * Retrieve the shortcode content
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function shortcode($atts) {
        global $wpdb;

        $atts = (object) $atts;
        if (false === empty($atts->name)) {
            $row = $wpdb->get_row("SELECT content FROM `{$wpdb->prefix}shortbus` WHERE name = '$atts->name' LIMIT 1");
            ob_start();
            echo '<div class="shortbus">' . eval('?>' . $row->content) . '</div>';
            return ob_get_clean();
        }
    }

    /*---------------------------------------------------------------------------------------------
     * Add the submenu under "Pages"
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_menu() {
        add_submenu_page('edit.php?post_type=page', 'Shortcodes', 'Shortcodes', 'edit_pages', 'shortbus', array($this, 'admin_page'));
    }

    /*---------------------------------------------------------------------------------------------
     * Save the shortcode data
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function handle_ajax() {
        global $wpdb;

        $id = (int) $_POST['id'];
        $method = $_POST['method'];

        if ('load' == $method) {
            $row = $wpdb->get_row("SELECT content FROM `{$wpdb->prefix}shortbus` WHERE id = '$id' LIMIT 1");
            echo htmlspecialchars($row->content);
        }
        elseif ('add' == $method) {
            $name = trim($_POST['name']);

            if (!preg_match('/^[A-Za-z0-9_ ]+$/', $name)) {
                echo json_encode(array(
                    'status' => 'error',
                    'data' => 'Please use only alphanumeric characters, spaces, and underscores.'
                ));
                die();
            }

            $wpdb->insert($wpdb->prefix . 'shortbus', array('name' => $name));
            echo json_encode(array(
                'status' => 'ok',
                'data' => '<option value="' . $wpdb->insert_id . '">' . $name . '</option>'
            ));
        }
        elseif ('edit' == $method) {
            $content = mysql_real_escape_string(stripslashes($_POST['content']), $wpdb->dbh);
            $wpdb->query("UPDATE `{$wpdb->prefix}shortbus` SET content = '$content' WHERE id = '$id' LIMIT 1");
            echo '[' . date_i18n('H:i:s') . '] Shortcode saved.';
        }
        elseif ('delete' == $method) {
            $wpdb->query("DELETE FROM `{$wpdb->prefix}shortbus` WHERE id = '$id' LIMIT 1");
        }
        die();
    }

    /*---------------------------------------------------------------------------------------------
     * Add javascript to the admin header
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_scripts() {
?>
<script type="text/javascript">
jQuery(function() {
    jQuery("#load-shortcode").change(function() {
        var data = {
            method: "load",
            action: "shortbus",
            id: jQuery(this).val()
        };
        if ("" == data.id) {
            jQuery("#shortcode-area").hide();
        }
        else {
            jQuery.post(ajaxurl, data, function(response) {
                jQuery("#shortcode-content").html(response);
                jQuery("#shortcode-area").show();
            });
        }
    });

    jQuery("#edit-shortcode").click(function() {
        var data = {
            method: "edit",
            action: "shortbus",
            id: jQuery("#load-shortcode option:selected").val(),
            content: jQuery("#shortcode-content").val()
        };
        jQuery.post(ajaxurl, data, function(response) {
            jQuery("#shortcode-response").show();
            jQuery("#shortcode-response").html("<p>"+response+"</p>");
        });
    });

    jQuery("#add-shortcode").click(function() {
        var data = {
            method: "add",
            action: "shortbus",
            name: jQuery("#shortcode-name").val()
        };

        if ("" == data.name) return;
        jQuery.post(ajaxurl, data, function(response) {
            if ("error" == response.status) {
                jQuery("#shortcode-response").show();
                jQuery("#shortcode-response").html("<p>"+response.data+"</p>");
            }
            else {
                jQuery("#shortcode-name").val("");
                jQuery("#shortcode-content").val("Your shortcode has been added. Enter some content for it here.");
                jQuery("#load-shortcode").append(response.data);
                jQuery("#load-shortcode option:last").attr("selected", "selected");
                jQuery("#shortcode-area").show();
            }
        }, "json");
    });

    jQuery("#delete-shortcode").click(function() {
        if (confirm("Are you sure you want to delete this shortcode?")) {
            var data = {
                method: "delete",
                action: "shortbus",
                id: jQuery("#load-shortcode option:selected").val()
            };
            jQuery.post(ajaxurl, data, function(response) {
                jQuery("#shortcode-area").hide();
                jQuery("#load-shortcode option:selected").remove();
            });
        }
    });
});
</script>
<?php
    }

    /*---------------------------------------------------------------------------------------------
     * Build the shortcode management page
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_page() {
        global $wpdb;
?>
<link href="<?php echo WP_PLUGIN_URL; ?>/shortbus/style.css" rel="stylesheet" type="text/css" />
<div class="wrap">
	<div id="icon-edit-pages" class="icon32 shortbus-icon"><br></div>
    <h2>Shortcodes</h2>

    <div id="shortcode-response" class="updated hidden"></div>

    <div style="margin:15px 0">
        <select id="load-shortcode">
            <option value="">- Select One -</option>
<?php
        $results = $wpdb->get_results("SELECT id, name FROM `{$wpdb->prefix}shortbus` ORDER BY name ASC");
        foreach ($results as $result) {
?>
            <option value="<?php echo $result->id; ?>"><?php echo $result->name; ?></option>
<?php
        }
?>
        </select>
        - or -
        <input id="shortcode-name" type="text" title="Enter a name for your shortcode" value="" />
        <a id="add-shortcode" class="button">Add New</a>
    </div>
    <div id="shortcode-area" class="hidden">
        <div><textarea id="shortcode-content"></textarea></div>
        <div style="margin-top:15px">
            <input name="save" type="submit" class="button-primary" id="edit-shortcode" value="Save Changes" />
            or <a id="delete-shortcode" href="javascript:;">Delete</a>
        </div>
    </div>
</div>
<?php
    }
}

/*---------------------------------------------------------------------------------------------
 * Initialize Shortbus
 *
 * @author Matt Gibbs
 * @since 1.0
 ---------------------------------------------------------------------------------------------*/
$sb = new Shortbus();
$sb->init();
